/*******************************************************************************

      i2c_slave.h

Authors:	   Adam Klaptocz (ak)

History:
	30.05.06/ak   Start day
	
Todo:

*******************************************************************************/
#ifndef I2C_SLAVE
#define I2C_SLAVE

#include "p18F6722.h"
#include "i2c.h"


// I2C status flags
#define WAITING 		0
#define WRITE_ADD 		1
#define WRITE_DATA		2
#define READ_STRING		3

#define BUFFER_SIZE		180

// internal I2C communication functions and 
void I2C_isr(void);
void init_I2C(char I2C_address);
unsigned char read_register(unsigned char reg_add);
void write_register(unsigned char reg_add, unsigned char value);

#pragma udata buffer
unsigned char buffer[BUFFER_SIZE];
#pragma code  // allow linker to locate remaining code
#pragma udata buffer_flag
unsigned char buffer_flag[BUFFER_SIZE];
#pragma code  // allow linker to locate remaining code

unsigned char address, mode, register_add, index;
#include "LED_turret.h"
#pragma interrupt I2C_isr
void I2C_isr(void)  // interrupt for receive
{
	if (!SSP1STATbits.D_A)		// current byte in buffer is module I2C address
	{
		address = SSP1BUF;		// clear I2C address from buffer
		if (SSP1STATbits.R_W)	// Master requesting read
		{
			//SSPCON1bits.CKP=0;	// hold clock
			SSP1BUF = read_register(register_add);  // define action to be taken during read in the function 'read_register'
			index = 1;			// set index for string read
			mode = READ_STRING;	// set mode for next interrupt
		}
		else					// Master sending write
			mode = WRITE_ADD;	// set mode for next interrupt
	}
	else		// current byte in buffer is data to be written
	{
		if (mode == WRITE_ADD)		// current byte is register address to be written
		{
			register_add = SSP1BUF;	// save register address
			mode = WRITE_DATA;		// set mode for next interrupt
			index = 0;				// set index for string write
		}
		else if (mode == WRITE_DATA)	// current byte is data to be written to register
		{
			write_register(register_add+index, SSP1BUF);  // define action to be taken during read in the function 'write_register'
			index++;				// increment string write index
		}
		else if (mode == READ_STRING) // ACK detected from Master, continue string read
		{
			SSP1BUF = read_register(register_add+index);  // define action to be taken during read in the function 'read_register'
			SSPCON1bits.CKP=1;							 // release clock			
			index++;									 // increment index for string read
		}
	}
	SSPCON1bits.CKP=1;	// release clock			
	PIR1bits.SSP1IF = 0;	// clear I2C interrupt flag
}
#pragma code  // allow linker to locate remaining code

void init_I2C(char I2C_address)
{
	SSPADD = I2C_address;		// set I2C address
	OpenI2C1(SLAVE_7,SLEW_OFF);
	
	RCONbits.IPEN = 1;		// enable priority levels on interrupts
	INTCONbits.GIEH = 1;	// global enable of high-priority interrupts
	INTCONbits.GIEL = 1;	// global enable of low-priority interrupts

	PIR1bits.SSP1IF = 0;	// clear SSP interrupt flag
	PIE1bits.SSP1IE = 1;	// enable SSP interrupt
	IPR1bits.SSP1IP = 1;	// set HIGH PRIORITY for SSP
	
	SSP1CON2bits.SEN = 1;	// enable clock stretching
	SSPCON1bits.CKP=1;	// release clock
}

unsigned char read_register(unsigned char reg_add)
{
	if (reg_add < BUFFER_SIZE)
		return buffer[reg_add];
	else
		return 0;
}

// write_register writes to command buffer
void write_register(unsigned char reg_add, unsigned char value)
{
	buffer[reg_add] = value;
	buffer_flag[reg_add] = 1;
}

#endif 
